"""
Interface to the GAP interpreter
"""

#*****************************************************************************
#       Copyright (C) 2005 William Stein <wstein@ucsd.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from expect import Expect
from sage.misc.misc import SAGE_HOME
import os
DB_HOME = "%s/data/"%SAGE_HOME
VERSION='4.4.6'

class GAP(Expect):
    def __init__(self, maxread=10000, script_subdirectory=""):

        empty_workspace = "%s/data/scripts/gap/empty_workspace"%SAGE_HOME
        if os.path.exists(empty_workspace):
            cmd = "gap -T -n -b -L %s"%empty_workspace
            make_workspace = False            
        else:
            cmd = "gap -T -n -b"
            make_workspace = True
        
        Expect.__init__(self,
                        name = 'gap',
                        prompt = 'gap> ',
                        command = cmd,
                        maxread = maxread,
                        script_subdirectory = script_subdirectory,
                        restart_on_ctrlc = True,
                        verbose_start = True)

        # This must go after the expect setup above, otherwise
        # we couldn't run this command in the interpreter.
        if make_workspace:
            self('SaveWorkspace("%s");'%empty_workspace)

    def __call__(self, x, newlines=True):
        x = x.rstrip()
        if len(x) == 0 or x[len(x) - 1] != ';':
            x += ';'
        s = Expect.__call__(self, x)
        if newlines:
            return s
        else:
            return s.replace("\\\n","")


running_gap = None            
def gap(s, newlines=True):
    r"""
    Send the code in the string s to the GAP interpreter and return
    the output as a string.

    INPUT:
        s -- string containing GAP code.
        newlines -- bool (default: True); if False, remove all
                  backslash-newlines inserted by the GAP output formatter.
    """
    global running_gap
    if running_gap is None:
        running_gap = GAP(script_subdirectory='user')
    return running_gap(s, newlines)

