"""
Interface to the gp interpreter
"""

#*****************************************************************************
#       Copyright (C) 2005 William Stein <wstein@ucsd.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from expect import Expect
from sage.misc.misc import verbose

class GP(Expect):
    def __init__(self, stacksize=50000000, maxread=10000, script_subdirectory=""):
        Expect.__init__(self,
                        name = 'gp',
                        prompt = '\\? ',
                        command = "gp --emacs --fast --quiet --stacksize %s"%stacksize,
                        maxread = maxread,
                        script_subdirectory = script_subdirectory,
                        restart_on_ctrlc = False,
                        verbose_start = False)

    def eval_line(self, line):
        a = Expect.eval_line(self, line)
        if a.find("the PARI stack overflows") != -1:
            verbose("doubling the PARI stack")
            self("allocatemem()")
            return self.eval_line(line)
        else:
            return a


instance = None            
def gp(x):
    """
    Send the code x to the PARI gp interpreter and return the output
    as a string.
    """
    global instance
    if instance is None:
        instance = GP(script_subdirectory='user')
    return instance(x)

